@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset: 4 -*-
  * libglade - a library for building interfaces from XML files at runtime
- * Copyright (C) 1998-2001  James Henstridge <james@daa.com.au>
+ * Copyright (C) 1998-2002  James Henstridge <james@daa.com.au>
  *
  * glade-parser.c: functions for parsing glade-2.0 files
  *
@@ -37,6 +37,7 @@
 #include <libxml/parser.h>
 
 #include "glade-parser.h"
+#include "glade-private.h"
 
 typedef enum {
     PARSER_START,
@@ -58,12 +59,14 @@ typedef enum {
     PARSER_WIDGET_CHILD_PACKING,
     PARSER_WIDGET_CHILD_PACKING_PROPERTY,
     PARSER_WIDGET_CHILD_AFTER_PACKING,
+    PARSER_WIDGET_CHILD_PLACEHOLDER,
+    PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER,
     PARSER_FINISH,
     PARSER_UNKNOWN
 } ParserState;
 
-#if 0
-static gchar *state_names[] = {
+#ifdef DEBUG
+static const gchar *state_names[] = {
     "START",
     "GLADE_INTERFACE",
     "REQUIRES",
@@ -83,8 +86,10 @@ static gchar *state_names[] = {
     "WIDGET_CHILD_PACKING",
     "WIDGET_CHILD_PACKING_PROPERTY",
     "WIDGET_CHILD_AFTER_PACKING",
+    "WIDGET_CHILD_PLACEHOLDER",
+    "WIDGET_CHILD_AFTER_PLACEHOLDER",
     "FINISH",
-    "UNKNOWN"
+    "UNKNOWN",
 };
 #endif
 
@@ -303,6 +308,8 @@ handle_signal(GladeParseState *state, const xmlChar **attrs)
 	    info.after = attrs[i+1][0] == 'y';
 	else if (!strcmp(attrs[i], "object"))
 	    info.object = alloc_string(state->interface, attrs[i+1]);
+	else if (!strcmp(attrs[i], "last_modification_time"))
+	    /* Do nothing. */;
 	else
 	    g_warning("unknown attribute `%s' for <signal>.", attrs[i]);
     }
@@ -473,6 +480,9 @@ glade_parser_start_element(GladeParseState *state,
 {
     int i;
 
+    GLADE_NOTE(PARSER, g_message("<%s> in state %s",
+				 name, state_names[state->state]));
+
     switch (state->state) {
     case PARSER_START:
 	if (!strcmp(name, "glade-interface")) {
@@ -545,23 +555,33 @@ glade_parser_start_element(GladeParseState *state,
 	break;
     case PARSER_WIDGET:
 	if (!strcmp(name, "property")) {
+	    gboolean bad_agent = FALSE;
+
 	    if (state->prop_type != PROP_NONE &&
 		state->prop_type != PROP_WIDGET)
 		g_warning("non widget properties defined here (oh no!)");
-	    state->prop_type = PROP_WIDGET;
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
 		    state->prop_name = alloc_string(state->interface,
 						    attrs[i+1]);
-		else if (!strcmp(attrs[i], "translatable") &&
-			 !strcmp(attrs[i+1], "yes"))
-		    state->translate_prop = TRUE;
+		else if (!strcmp(attrs[i], "translatable"))
+		    state->translate_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "agent"))
+		    bad_agent = strcmp(attrs[i], "libglade") != 0;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
 	    }
-	    state->state = PARSER_WIDGET_PROPERTY;
+	    if (bad_agent) {
+		/* ignore the property ... */
+		state->prev_state = state->state;
+		state->state = PARSER_UNKNOWN;
+		state->unknown_depth++;
+	    } else {
+		state->prop_type = PROP_WIDGET;
+		state->state = PARSER_WIDGET_PROPERTY;
+	    }
 	} else if (!strcmp(name, "accessibility")) {
 	    flush_properties(state);
 
@@ -601,9 +621,8 @@ glade_parser_start_element(GladeParseState *state,
 		if (!strcmp(attrs[i], "name"))
 		    state->prop_name = alloc_string(state->interface,
 						    attrs[i+1]);
-		else if (!strcmp(attrs[i], "translatable") &&
-			 !strcmp(attrs[i+1], "yes"))
-		    state->translate_prop = TRUE;
+		else if (!strcmp(attrs[i], "translatable"))
+		    state->translate_prop = !strcmp(attrs[i+1], "yes");
 		else
 		    g_warning("unknown attribute `%s' for <atkproperty>.",
 			      attrs[i]);
@@ -718,6 +737,10 @@ glade_parser_start_element(GladeParseState *state,
 	    state->accels = NULL;
 
 	    state->state = PARSER_WIDGET;
+	} else if (!strcmp(name, "placeholder")) {
+	    /* this isn't a real child, so knock off  the last ChildInfo */
+	    state->widget->n_children--;
+	    state->state = PARSER_WIDGET_CHILD_PLACEHOLDER;
 	} else {
 	    g_warning("Unexpected element <%s> inside <child>.", name);
 	    state->prev_state = state->state;
@@ -737,23 +760,33 @@ glade_parser_start_element(GladeParseState *state,
 	break;
     case PARSER_WIDGET_CHILD_PACKING:
 	if (!strcmp(name, "property")) {
+	    gboolean bad_agent = FALSE;
+
 	    if (state->prop_type != PROP_NONE &&
 		state->prop_type != PROP_CHILD)
 		g_warning("non child properties defined here (oh no!)");
-	    state->prop_type = PROP_CHILD;
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
 		    state->prop_name = alloc_string(state->interface,
 						    attrs[i+1]);
-		else if (!strcmp(attrs[i], "translatable") &&
-			 !strcmp(attrs[i+1], "yes"))
-		    state->translate_prop = TRUE;
+		else if (!strcmp(attrs[i], "translatable"))
+		    state->translate_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "agent"))
+		    bad_agent = strcmp(attrs[i], "libglade") != 0;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
 	    }
-	    state->state = PARSER_WIDGET_CHILD_PACKING_PROPERTY;
+	    if (bad_agent) {
+		/* ignore the property ... */
+		state->prev_state = state->state;
+		state->state = PARSER_UNKNOWN;
+		state->unknown_depth++;
+	    } else {
+		state->prop_type = PROP_CHILD;
+		state->state = PARSER_WIDGET_CHILD_PACKING_PROPERTY;
+	    }
 	} else {
 	    g_warning("Unexpected element <%s> inside <child>.", name);
 	    state->prev_state = state->state;
@@ -773,6 +806,18 @@ glade_parser_start_element(GladeParseState *state,
 	state->state = PARSER_UNKNOWN;
 	state->unknown_depth++;
 	break;
+    case PARSER_WIDGET_CHILD_PLACEHOLDER:
+	g_warning("<placeholder> should be empty.  Found <%s>.", name);
+	state->prev_state = state->state;
+	state->state = PARSER_UNKNOWN;
+	state->unknown_depth++;
+	break;
+    case PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER:
+	/* this is a placeholder <child> element -- ignore extra elements */
+	state->prev_state = state->state;
+	state->state = PARSER_UNKNOWN;
+	state->unknown_depth++;
+	break;
     case PARSER_FINISH:
 	g_warning("There should be no elements here.  Found <%s>.", name);
 	state->prev_state = state->state;
@@ -791,6 +836,10 @@ static void
 glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 {
     GladeProperty prop;
+
+    GLADE_NOTE(PARSER, g_message("</%s> in state %s",
+				 name, state_names[state->state]));
+
     switch (state->state) {
     case PARSER_START:
 	g_warning("should not be closing any elements in this state");
@@ -831,7 +880,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop) {
+	if (state->translate_prop && state->content->str[0] != '\0') {
 	    prop.value = alloc_string(state->interface,
 			dgettext(state->domain, state->content->str));
 	} else {
@@ -853,7 +902,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop) {
+	if (state->translate_prop && state->content->str[0] != '\0') {
 	    prop.value = alloc_string(state->interface,
 			dgettext(state->domain, state->content->str));
 	} else {
@@ -915,7 +964,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop) {
+	if (state->translate_prop && state->content->str[0] != '\0') {
 	    prop.value = alloc_string(state->interface,
 			dgettext(state->domain, state->content->str));
 	} else {
@@ -930,6 +979,16 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	    g_warning("should find </child> here.  Found </%s>", name);
 	state->state = PARSER_WIDGET_AFTER_ACCEL;
 	break;
+    case PARSER_WIDGET_CHILD_PLACEHOLDER:
+	if (strcmp(name, "placeholder") != 0)
+	    g_warning("should find </placeholder> here.  Found </%s>", name);
+	state->state = PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER;
+	break;
+    case PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER:
+	if (strcmp(name, "child") != 0)
+	    g_warning("should find </child> here.  Found </%s>", name);
+	state->state = PARSER_WIDGET_AFTER_ACCEL;
+	break;
     case PARSER_FINISH:
 	g_warning("should not be closing any elements in this state");
 	break;
@@ -1101,6 +1160,9 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
     }
     if (state.state != PARSER_FINISH) {
 	g_warning("did not finish in PARSER_FINISH state!");
+	if (state.interface)
+	    glade_interface_destroy(state.interface);
+	return NULL;
     }
     return state.interface;
 }
@@ -1139,6 +1201,9 @@ glade_parser_parse_buffer(const gchar *buffer, gint len, const gchar *domain)
     }
     if (state.state != PARSER_FINISH) {
 	g_warning("did not finish in PARSER_FINISH state!");
+	if (state.interface)
+	    glade_interface_destroy(state.interface);
+	return NULL;
     }
     return state.interface;
 }
