@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset: 4 -*-
  * libglade - a library for building interfaces from XML files at runtime
- * Copyright (C) 1998-2001  James Henstridge <james@daa.com.au>
+ * Copyright (C) 1998-2002  James Henstridge <james@daa.com.au>
  *
  * glade-xml.h: public interface of libglade library
  *
@@ -65,10 +65,10 @@ gboolean   glade_xml_construct           (GladeXML *self, const char *fname,
 
 void       glade_xml_signal_connect      (GladeXML *self,
 					  const char *handlername,
-					  GtkSignalFunc func);
+					  GCallback func);
 void       glade_xml_signal_connect_data (GladeXML *self,
 					  const char *handlername,
-					  GtkSignalFunc func,
+					  GCallback func,
 					  gpointer user_data);
 /*
  * use gmodule to connect signals automatically.  Basically a symbol with
@@ -82,10 +82,10 @@ void       glade_xml_signal_autoconnect  (GladeXML *self);
  * will point to the object, otherwise it will be NULL.
  */
 typedef void (*GladeXMLConnectFunc) (const gchar *handler_name,
-				     GtkObject *object,
+				     GObject *object,
 				     const gchar *signal_name,
 				     const gchar *signal_data,
-				     GtkObject *connect_object,
+				     GObject *connect_object,
 				     gboolean after,
 				     gpointer user_data);
 
